@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: CardBox.cpp,v 1.47 2005/11/13 02:50:48 vlg Exp $
+// $Id: CardBox.cpp,v 1.50 2006/02/20 18:23:17 vlg Exp $
 //------------------------------------------------------------------------------
 //                            CardBox.cpp
 //------------------------------------------------------------------------------
@@ -40,7 +40,7 @@ CardBox::
 calculate_relative_path (const std::string& abs_pname_,
 						 const std::string& abs_fname_)
 {
-    trace_with_mask("CardBox::calculate_relative_path", GUITRACE);
+//    trace_with_mask("CardBox::calculate_relative_path", GUITRACE);
 
 	std::string::iterator ap_iter;
 	std::string::iterator af_iter;
@@ -538,8 +538,8 @@ save_as (const string& fname_)
 	if (Glib::file_test (fname_, Glib::FILE_TEST_IS_REGULAR)) {
 		string msg = N_("File \n") + fname_ 
 			+ _("\nalready exists.\nWould you like to delete it first?");
-		Gtk::MessageDialog qm (*MAINWIN, 
-							   msg, 
+		/* Gtk::MessageDialog qm (*MAINWIN,  */
+		Gtk::MessageDialog qm (msg, 
 							   false,
 							   Gtk::MESSAGE_QUESTION, 
 							   Gtk::BUTTONS_NONE,
@@ -709,9 +709,17 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 	DL((GRAPP,"Placing card in next/first CardDeck ...\n"));
 	vcard->set_expiration (CONFIG->get_expiration (to_deck_));
 	m_card_deck [to_deck_].push_back (vcard);
+
+	if (GRANULE->dump_cardboxes_enabled ()) {
+		for (int j = 0; j < CARD_BOX_SIZE; j++) {
+			m_card_deck [j].dump_to_file ();
+		}
+	}
+
 	if (to_deck_ != from_deck_) {
 		m_card_deck [to_deck_].reset_progress ();
 	}
+
 	m_cdplayers [to_deck_]->reset ();
 	update_count (to_deck_);
 	set_expiration_light (to_deck_);
@@ -806,8 +814,8 @@ on_deck_dropped_cb (const Glib::RefPtr<Gdk::DragContext>& context_,
 	os << "Proceed with adding Deck \n\"" << src_deck->get_name () << "\"\n"
 	   << " to CardBox #" << (card_deck_index_+1) << " ?";
 
-	Gtk::MessageDialog qm (*MAINWIN,
-						   _ (os.str ().c_str ()),
+	/* Gtk::MessageDialog qm (*MAINWIN, */
+	Gtk::MessageDialog qm (_ (os.str ().c_str ()),
 						   false,
 						   Gtk::MESSAGE_QUESTION, 
 						   Gtk::BUTTONS_NONE,
@@ -838,3 +846,13 @@ repaint_deck_players ()
 	}
 }
 
+void
+CardBox::
+export_sound_bits (const std::string& path_)
+{
+	trace_with_mask("CardBox::export_sound_bits",GUITRACE);
+
+	for (int i = 0; i < CARD_BOX_SIZE; i++) {
+		m_card_deck [i].export_sound_bits (path_);
+	}
+}
