@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.cpp
 //------------------------------------------------------------------------------
-// $Id: CardView.cpp,v 1.28 2005/11/27 23:54:36 vlg Exp $
+// $Id: CardView.cpp,v 1.33 2006/03/19 15:39:44 vlg Exp $
 //
 //  Copyright (c) 2004-2005 by Vladislav Grinchenko
 //
@@ -36,6 +36,9 @@
 
 CardView::
 CardView (VCard* card_) : 
+#ifdef IS_HILDON
+	Hildon::AppView ("Card View"),
+#endif
 	m_card (card_),
 	m_cancel_button (NULL),
 	m_add_n_close_button (NULL),
@@ -93,6 +96,9 @@ CardView (VCard* card_) :
 	Gtk::Table* elems_table;
 	Gtk::VBox*  vbox_all;
 
+#ifdef IS_HILDON
+	set_size_request (672, 396);
+#else
 	set_icon_from_file (GRAPPDATADIR "/pixmaps/cardview_32x32.png");
 
 	Gdk::Rectangle geom = CONFIG->get_card_view_geometry ();
@@ -103,6 +109,7 @@ CardView (VCard* card_) :
 	set_modal (true);
 	set_transient_for (*MAINWIN);
 	property_window_position ().set_value (Gtk::WIN_POS_CENTER);
+#endif
 
 	/** <Clear> button
 	 */
@@ -335,6 +342,14 @@ CardView (VCard* card_) :
 									   &EditControls::on_focus_changed),
 							  m_example_text));
 
+#ifdef IS_HILDON
+	/** Trap 'on virtual keyboard close' event to restore
+	 *  original size of the dialog
+	 */
+	HILDONAPPWIN->signal_im_close ().connect (
+		mem_fun (*this, &CardView::hildon_im_close_cb));
+#endif
+
 	/** Front Text
 	 */
 	m_front_text->set_flags (Gtk::CAN_FOCUS);
@@ -401,6 +416,24 @@ CardView (VCard* card_) :
 	elems_table->attach (*alignment7, 0, 1, 3, 4, 
 						 Gtk::FILL, Gtk::FILL, 0, 0);
 
+#ifdef IS_HILDON
+    /** Maemo has a narrow screen and thus requires
+	 *  smart packing.
+	 */
+	Gtk::ScrolledWindow* scrollw = Gtk::manage (new Gtk::ScrolledWindow);
+	scrollw->set_flags (Gtk::CAN_FOCUS);
+	scrollw->set_shadow_type (Gtk::SHADOW_NONE);
+	scrollw->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
+	scrollw->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
+
+	scrollw->add (*elems_table);
+	vbox_all->pack_start (*scrollw);
+#else                                      // Regular desktop 
+	vbox_all->pack_start (*elems_table);
+#endif
+
+	/** Create and pack buttons
+	 */
 	Gtk::HBox* hbbox = Gtk::manage(new Gtk::HBox(false, 0));
 	Gtk::HSeparator* hseparator = manage (new Gtk::HSeparator);
 	Gtk::HButtonBox*  hbuttonbox = manage (new Gtk::HButtonBox);
@@ -416,12 +449,13 @@ CardView (VCard* card_) :
 	hbbox->pack_start (*m_help_button, Gtk::PACK_SHRINK, 5);
 	hbbox->pack_start (*hbuttonbox);
 
-	vbox_all->pack_start (*elems_table);
 	vbox_all->pack_start (*hseparator, Gtk::PACK_SHRINK, 3);
 	vbox_all->pack_start (*hbbox, Gtk::PACK_SHRINK, 5);
 
 	add (*vbox_all);
 
+	/** Setup signals
+	 */
 	m_help_button->signal_clicked ().connect (
 		sigc::mem_fun (*this, &CardView::on_help_clicked));
 
@@ -482,7 +516,14 @@ run (bool disable_cancel_)
 		m_cancel_button->set_sensitive (false);
 	}
 	take_snapshot ();
+
+#ifdef IS_HILDON
+	GRANULE->register_appview (this);
+	Gtk::Main::run (*HILDONAPPWIN);
+#else
 	Gtk::Main::run (*this);
+#endif
+
 	return (m_syntax_error ? Gtk::RESPONSE_REJECT : m_response);
 }
 
@@ -505,32 +546,49 @@ CardView::
 run (DeckView& deckview_)
 {
 	trace_with_mask("CardView::run(DeckView&)",GUITRACE);
+
 	int ret = 0;
 	m_syntax_error = false;
 	take_snapshot ();
 
+#ifdef IS_HILDON
+	GRANULE->register_appview (this);
+	Gtk::Main::run (*HILDONAPPWIN);
+#else
+	set_transient_for (deckview_);
 	Gtk::Main::run (*this);
+#endif
 
+	DL ((GRAPP,"Processing syntax_error and response...\n"));
 	if (m_syntax_error) {
 		DL ((GRAPP,"Caught a syntax error (RESPONSE_REJECT)\n"));
 		delete m_card;
-		return (Gtk::RESPONSE_REJECT);
+		m_response = Gtk::RESPONSE_REJECT;
+		goto done;
 	}
 
 	if (m_response == Gtk::RESPONSE_CANCEL) {
 		DL ((GRAPP,"Dialog cancelled (RESPONSE_CANCEL)\n"));
 		delete m_card;
-		return (m_response);
+		goto done;
 	}
 
 	deckview_.add_new_card (dynamic_cast<Card*>(m_card));
 
-	while (m_keep_open) {
+	while (m_keep_open) 
+	{
+		DL ((GRAPP,"Looping ...\n"));
 		m_card = new Card;
+
+#ifdef IS_HILDON
+		GRANULE->register_appview (this);
+		Gtk::Main::run (*HILDONAPPWIN);
+#else
 		Gtk::Main::run (*this);
+#endif
 		if (m_syntax_error) {
-			m_response = Gtk::RESPONSE_REJECT;
 			delete m_card;
+			m_response = Gtk::RESPONSE_REJECT;
 			break;
 		}
 		if (m_response == Gtk::RESPONSE_CANCEL) {
@@ -539,9 +597,27 @@ run (DeckView& deckview_)
 		}
 		deckview_.add_new_card (dynamic_cast<Card*>(m_card));
 	}
+
+  done:
 	return (m_response);
 }
 
+void
+CardView::
+on_cancel_clicked  ()
+{
+	trace_with_mask("CardView::on_cancel_clicked",GUITRACE);
+	
+	m_response = Gtk::RESPONSE_CANCEL;
+
+#ifdef IS_HILDON
+	GRANULE->unregister_appview (this);
+#endif
+
+	DL ((GRAPP,"Call Main::quit()\n"));
+	Gtk::Main::quit ();
+}
+
 /** Fend off clicking Esc key on the keyboard.
 	This often happens when I want to enter ~ (tilda) key, and instead
 	press Esc. The effect is to close the editing window which leads to
@@ -564,20 +640,17 @@ on_key_press_event (GdkEventKey* event_)
 	if (event_->keyval == GDK_Escape) { 
 		return false;
 	}
-	return Window::on_key_press_event (event_);	// keep processing keystroke
-}
-	   
 
-void
-CardView::
-on_cancel_clicked  ()
-{
-	trace_with_mask("CardView::on_cancel_clicked",GUITRACE);
-	
-	m_response = Gtk::RESPONSE_CANCEL;
-	Gtk::Main::quit ();
+    /** Keep processing keystroke
+	 */
+#ifdef IS_HILDON
+	return Bin::on_key_press_event (event_);	
+#else
+	return Window::on_key_press_event (event_);
+#endif
 }
 
+
 void
 CardView::
 on_addnclose_clicked  ()
@@ -622,8 +695,15 @@ on_addnclose_clicked  ()
 	}
 	m_keep_open = false;
 	m_front_text->grab_focus ();
-	
+
+#ifdef IS_HILDON
+	GRANULE->unregister_appview (this);
+
+	DL ((GRAPP,"Call Main::quit()\n"));
 	Gtk::Main::quit ();
+#else	
+	Gtk::Main::quit ();
+#endif
 }
 
 void
@@ -690,3 +770,13 @@ size_allocate_cb (Gtk::Allocation& allocation_)
 									allocation_.get_height ());
 }
 
+#ifdef IS_HILDON
+void
+CardView::
+hildon_im_close_cb ()
+{
+    trace_with_mask("CardView(hildon)::hildon_im_close_cb",GUITRACE);
+
+	set_size_request (672, 396);
+}
+#endif
