@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Config.cpp
 //------------------------------------------------------------------------------
-// $Id: Config.cpp,v 1.37 2005/11/22 01:39:53 vlg Exp $
+// $Id: Config.cpp,v 1.39 2006/02/10 03:26:05 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004-2005 by Vladislav Grinchenko
 //
@@ -95,17 +95,24 @@ Config () :
     m_path_name = ptr;
     delete [] ptr;
 
-/*  If fonts description is missing, you might find out 
-	theme's default font.
-
-	Glib::RefPtr<Gtk::Style> style = m_example.get_style ();
-	Pango::FontDescription default_font = style->get_font ();
-	Glib::ustring usfont = default_font.to_string ();
-	DL((APP, "Default font is \"%s\"\n", usfont.c_str ()));
-*/
 	set_question_font ("Sans 14");
 	set_answer_font   ("Sans 14");
 	set_example_font  ("Sans 14");
+	set_input_font    ("Sans 12");
+
+	/**  If fonts description is missing, you need to find out the current
+	 *	 theme's default font. Maemo default theme has overblown fonts,
+	 *	 so we scale them down to fit windows in 800x480 resolution.
+
+	 Glib::RefPtr<Gtk::Style> style = m_example.get_style ();
+	 Pango::FontDescription default_font = style->get_font ();
+	 set_app_font (default_font.to_string ());
+
+	 */
+
+#ifdef IS_HILDON
+	set_app_font ("Bitstream Vera Sans 12");
+#endif
 
 	/** Get Real User name. Retrieved from /etc/passwd, it comes in a format
 		'John Doe,,301-JOH-NDOE'.
@@ -273,6 +280,8 @@ remove_document_history (const string& fullpath_)
  *    question_font_desc="Sans 14"
  *    answer_font_desc="Sans 14"
  *    example_font_desc="Sans 14"
+ *    input_font_desc="Sans 14"
+ *    app_font_desc="Bitstream Vera Sans 12"
  *    mainwin_geometry=430x350
  *    cardview_geometry_height=643x386
  *    deckplayer_geometry=583x350
@@ -443,6 +452,16 @@ load_config (int secs_in_day_, int secs_in_week_)
 		set_example_font (usfont);
     }
 
+    usfont = m_config->get_value ("Default", "input_font_desc");
+    if (! usfont.empty ()) {
+		set_input_font (usfont);
+    }
+
+    usfont = m_config->get_value ("Default", "app_font_desc");
+    if (! usfont.empty ()) {
+		set_app_font (usfont);
+    }
+
     s = m_config->get_value ("Default", "mainwin_geometry");
     if (! s.empty ()) {
 		str_to_geometry (s, m_mwin_geometry);
@@ -582,6 +601,12 @@ save_config ()
     usfont = m_example_font_desc.to_string ();
     m_config->set_pair ("Default", 
 						IniFile::tuple_type("example_font_desc",usfont));
+    usfont = m_input_font_desc.to_string ();
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("input_font_desc",usfont));
+    usfont = m_app_font_desc.to_string ();
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("app_font_desc",usfont));
 
     /** Geometry
      */
