@@ -2,9 +2,9 @@
 //------------------------------------------------------------------------------
 //                              Deck.cpp
 //------------------------------------------------------------------------------
-// $Id: Deck.cpp,v 1.41 2005/11/13 02:50:48 vlg Exp $
+// $Id: Deck.cpp,v 1.44 2006/03/06 03:59:38 vlg Exp $
 //
-//  Copyright (c) 2004-2005 by Vladislav Grinchenko
+//  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -18,6 +18,11 @@
 using std::ofstream;
 using std::ifstream;
 
+#ifdef IS_HILDON
+#  include <glib.h>
+#  include <hildon-widgets/hildon-system-sound.h>
+#endif
+
 #include <assa/CommonUtils.h>
 #include <assa/Fork.h>
 #include <assa/TimeVal.h>
@@ -77,6 +82,7 @@ load (const string& fname_, DeckAction action_)
 	string error_msg;
 	xmlParserCtxtPtr context;
 	xmlDocPtr parser;
+	int ret;
 
 	context = xmlCreateFileParserCtxt (fname_.c_str ()); // new
 
@@ -118,11 +124,13 @@ load (const string& fname_, DeckAction action_)
 
 	DL((GRAPP,"Parsing the deck ...\n"));
 
-	if (parse_xml_tree (parser, error_msg) < 0) {
+	ret = parse_xml_tree (parser, error_msg);
+	xmlFreeDoc (parser);
+
+	if (ret < 0) {
 		DECKMGR->on_deck_loaded (this, LOAD_FAILED);
 		Gtk::MessageDialog emsg (error_msg, MESSAGE_ERROR);
 		emsg.run ();
-		xmlFreeDoc (parser);
 		return -1;
 	}
 
@@ -132,7 +140,6 @@ load (const string& fname_, DeckAction action_)
 	m_status = CLEAN_DECK;
 	dump_deck_status ();
 
-	xmlFreeDoc (parser);
 	return 0;
 }
 
@@ -210,20 +217,16 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 
 		sprintf (xpath, "/deck/card[%d]/back", cards_counter);
 		result = Granule::get_node_by_xpath (parser_, xpath);
-		if (result == NULL) {
-			os << "<back> tag is missing in card # " << cards_counter
-			   << "\nof the deck " << m_fname;
-			error_msg_ = os.str ();
-			return -1;
+		if (result != NULL) {	                              /* can be empty */
+			card->set_answer ((const char*) result);
+			xmlFree (result);
 		}
-		card->set_answer ((const char*) result);
-		xmlFree (result);
 
 		/** back_example node might be empty - thus non-critical.
 		 */
 		sprintf (xpath, "/deck/card[%d]/back_example", cards_counter);
 		result = Granule::get_node_by_xpath (parser_, xpath);
-		if (result) {
+		if (result != NULL) {                                 /* can be empty */
 			card->set_example ((const char*) result);
 			xmlFree (result);
 		}
@@ -411,7 +414,7 @@ save_as (const string& fname_)
 	if (Glib::file_test (fname_ext, Glib::FILE_TEST_IS_REGULAR)) {
 		string msg = "File \n" + fname_ext
 			+ "\nalready exists.\nWould you like to delete it first?";
-		Gtk::MessageDialog qm (*MAINWIN, msg, false,
+		Gtk::MessageDialog qm (msg, false,
 							   Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE,
 							   true);
 		qm.add_button ("Yes", Gtk::RESPONSE_YES);
@@ -538,13 +541,11 @@ VCard*
 Deck::
 find_card_by_id (long id_)
 {
-	trace_with_mask("Deck::find_card_by_id",GUITRACE|DECK);
+//	trace_with_mask("Deck::find_card_by_id",GUITRACE|DECK);
 
 	cardlist_iterator iter = m_cards.begin ();
 	while (iter != m_cards.end ()) {
-		DL ((GRAPP,"Trying id=%ld\n", (*iter)->get_id ()));
 		if ((*iter)->get_id () == id_) {
-			DL ((GRAPP,"... that's a match\n"));
 			return (*iter);
 		}
 		iter++;
@@ -554,7 +555,7 @@ find_card_by_id (long id_)
 
 /** Rules for playable name are as follows:
 
-    1) A word can be preceeded by "to " to indicate it is a verb.
+    1) A word can be preceeded by "to " to indicate that it is a verb.
 	2) All multiword expressions are ignored.
 	3) The WAV file is assumed to be stored in the directory
        with the name of its first letter, i.e. "granule" will be
@@ -591,6 +592,12 @@ play_card (Deck::cardlist_iterator& iter_)
 		DL((APP,"Can't stat sound file \"%s\"\n", sfn.c_str ()));
 		return;
 	}
+
+#ifdef IS_HILDON
+	hildon_play_system_sound (sfn.c_str ());
+	DL((APP,"Playing \"%s\" WAV\n", sfn.c_str ()));
+#else
+
 	snprintf (execmdbuf, 512, CONFIG->get_snd_player_args (), sfn.c_str ());
 	DL((APP,"Playing \"%s %s\"\n", CONFIG->get_snd_player_cmd (), execmdbuf));
 
@@ -604,6 +611,7 @@ play_card (Deck::cardlist_iterator& iter_)
 	*/
 	ASSA::Fork::fork_exec (CONFIG->get_snd_player_cmd (), 
 						   execmdbuf, ASSA::Fork::COLLECT_STATUS);
+#endif
 }
 
 /** 
