@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckPlayer.cpp,v 1.83 2005/11/22 01:39:53 vlg Exp $
+// $Id: DeckPlayer.cpp,v 1.90 2006/03/19 15:39:44 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckPlayer.cpp
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004-2005 by Vladislav Grinchenko 
+//  Copyright (c) 2004-2006 by Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -38,7 +38,6 @@ using sigc::mem_fun;
 #include "Granule-main.h"
 #include "Granule.h"
 #include "MainWindow.h"
-#include "Config.h"
 #include "Card.h"
 #include "Deck.h"
 #include "CardBox.h"
@@ -100,6 +99,7 @@ VerifyControl (DeckPlayer& parent_) :
     m_entry->set_has_frame   (true);
     m_entry->set_width_chars (30);
     m_entry->set_activates_default (false);
+	m_entry->modify_font     (CONFIG->get_input_font ());
 
 	for (int i = 0; i < LABELS_SZ; i++) {
 		m_label [i] = Gtk::manage (new Gtk::Label (clabels [i]));
@@ -147,6 +147,14 @@ VerifyControl (DeckPlayer& parent_) :
         mem_fun (m_parent, &DeckPlayer::on_verify_clicked));
 }
 
+void
+VerifyControl::
+set_focus_on_entry () 
+{ 
+    trace_with_mask("VerifyControl::set_focus_on_entry", GUITRACE);
+	m_entry->grab_focus (); 
+}
+
 void
 VerifyControl::
 on_test_line_changed ()
@@ -279,12 +287,12 @@ clear ()
 	m_report_box->modify_fg   (Gtk::STATE_INSENSITIVE, m_black);
 
 	m_entry->set_text ("");
-	m_entry->grab_focus ();
 	m_entry->set_editable (true);
 
 	m_button->set_label (clabels [CHECK_ANSWER]);
 	m_button->set_sensitive (false);
 
+	set_focus_on_entry ();
 	m_parent.enable_answer_controls ();
 }
 
@@ -314,7 +322,12 @@ Gtk::StockID DeckPlayer::THUMB_UP ("thumb-up");
 Gtk::StockID DeckPlayer::THUMB_DOWN ("thumb-down");
 
 DeckPlayer::
-DeckPlayer (VDeck& vdeck_) :
+DeckPlayer (VDeck& vdeck_) 
+		:
+#ifdef IS_HILDON
+	Hildon::AppView ("Deck Player"),
+	m_hildon_fullscreen (false),
+#endif
     m_close_button      (
 		manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, _("Close")))),
     m_shuffle_button    (
@@ -341,6 +354,20 @@ DeckPlayer (VDeck& vdeck_) :
     trace_with_mask("DeckPlayer::DeckPlayer", GUITRACE);
 
     set_title ("DeckPlayer");
+
+#ifdef IS_HILDON
+	/**
+	 * Not a dialog any more - it is a Bin (AppView) instead.
+	 * We handle packaging ourselves and mute all attributes of a 
+	 * Dialog everywhere down the line.
+	 */
+    m_vbox = manage (new Gtk::VBox (false, 2));
+	add (*m_vbox);
+
+#else
+	/**
+	 * A desktop Dialog - we can resize and move the window around
+	 */	
     set_resizable (true);
 
 	Gdk::Rectangle geom = CONFIG->get_deck_player_geometry ();
@@ -348,8 +375,11 @@ DeckPlayer (VDeck& vdeck_) :
 
     set_position (Gtk::WIN_POS_CENTER_ALWAYS);
     set_modal (false);
+
 	set_transient_for (*MAINWIN);
 
+#endif
+
     /** Are we dealing with CardDeck?
      */
     if (dynamic_cast<Deck*>(&m_deck) == NULL) {
@@ -376,6 +406,7 @@ DeckPlayer (VDeck& vdeck_) :
     m_answer_box = manage (new Gtk::VBox (false, 5));
     m_answer_box->set_border_width (5); // How far box is placed from the edges.
 
+#ifndef IS_HILDON
 	if (CONFIG->keep_deck_player_aspect ()) {
 		Gdk::Rectangle& rec = CONFIG->get_deck_player_aspect ();
 		gdouble aspect = rec.get_width () * 1.0 / rec.get_height ();
@@ -390,13 +421,15 @@ DeckPlayer (VDeck& vdeck_) :
 		set_geometry_hints (*this, answer_box_geometry, 
 							Gdk::HINT_ASPECT | Gdk::HINT_MIN_SIZE);
 	}
+#endif
 
     /** Fill up notebook with elements; first front of the card
      */
-    m_front.set_alignment (0.46, 0.5);
-
-    m_front.set_justify    (Gtk::JUSTIFY_LEFT);
+#ifndef IS_HILDON
     m_front.set_selectable ();
+#endif
+    m_front.set_alignment (0.46, 0.5);
+    m_front.set_justify    (Gtk::JUSTIFY_LEFT);
     m_front.set_use_markup ();
 	m_front.set_line_wrap  (true);
     m_front.modify_font    (CONFIG->get_question_font ());
@@ -421,7 +454,9 @@ DeckPlayer (VDeck& vdeck_) :
 
     /* -back- */
     m_back.set_justify    (Gtk::JUSTIFY_FILL);
+#ifndef IS_HILDON
     m_back.set_selectable ();
+#endif
     m_back.set_use_markup ();
     m_back.modify_font    (CONFIG->get_answer_font ());
     m_back.set_line_wrap  ();
@@ -430,7 +465,9 @@ DeckPlayer (VDeck& vdeck_) :
     m_example.set_padding    (6, 0);  /* x - distance between text and 
 										 right/left edges */
     m_example.set_justify    (Gtk::JUSTIFY_LEFT);
+#ifndef IS_HILDON
     m_example.set_selectable ();
+#endif
     m_example.set_use_markup ();
     m_example.modify_font    (CONFIG->get_example_font ());
     m_example.set_line_wrap  ();
@@ -439,33 +476,50 @@ DeckPlayer (VDeck& vdeck_) :
     m_answer_box->pack_start (m_back,     Gtk::PACK_SHRINK, 5);
     m_answer_box->pack_start (*separator, Gtk::PACK_SHRINK, 5);
     m_answer_box->pack_start (m_example,  Gtk::PACK_SHRINK, 5); 
+    m_answer_box->show ();
 
+#ifdef IS_HILDON
+
+	Gtk::ScrolledWindow* scrollw = Gtk::manage (new Gtk::ScrolledWindow);
+
+	scrollw->set_flags (Gtk::CAN_FOCUS);
+	scrollw->set_shadow_type (Gtk::SHADOW_NONE);
+	scrollw->set_policy (Gtk::POLICY_NEVER , Gtk::POLICY_AUTOMATIC);
+	scrollw->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
+
+	scrollw->add (*m_answer_box);
+    m_notebook->pages ().push_back (
+		Gtk::Notebook_Helpers::TabElem (*scrollw, _("Back")));
+#else
     m_notebook->pages ().push_back (
 		Gtk::Notebook_Helpers::TabElem (*m_answer_box, _("Back")));
-    m_answer_box->show ();
+#endif
 
     /** Side-panel controls 
      */
     Gtk::VBox* vbox = manage (new Gtk::VBox (false, 5));
 
     Gtk::RadioButton::Group group;
-    Gtk::RadioButton* radio_fb;
-    Gtk::RadioButton* radio_bb;
+    Gtk::RadioButton* radio_fb = NULL;
+    Gtk::RadioButton* radio_bb = NULL;
     Gtk::VButtonBox*  vbuttonbox = manage (new Gtk::VButtonBox);
+
+#ifndef IS_HILDON
     vbuttonbox->set_homogeneous ();
     vbuttonbox->set_spacing (1);
+#endif	
                                                                                 
     radio_fb = manage (new Gtk::RadioButton (group, _("Ask front")));
     radio_fb->signal_clicked ().connect (
 		bind<SideSelection>(
 			mem_fun (*this, &DeckPlayer::on_radio_selected), FRONT));
-    vbuttonbox->pack_end (*radio_fb, false, false, 1);
+    vbuttonbox->pack_end (*radio_fb, false, false, 1); 
                                                                                 
     radio_bb = manage (new Gtk::RadioButton (group, _("Ask back")));
     radio_bb->signal_clicked ().connect (
 		bind<SideSelection>(
 			mem_fun (*this, &DeckPlayer::on_radio_selected), BACK));
-    vbuttonbox->pack_end (*radio_bb, false, false, 1);
+    vbuttonbox->pack_end (*radio_bb, false, false, 1); 
     vbox->pack_start (*vbuttonbox, false, false, 0);
 
 	/** Add Line selection spin button
@@ -481,6 +535,7 @@ DeckPlayer (VDeck& vdeck_) :
 	m_test_line_button->signal_value_changed ().connect (
 		mem_fun (*m_verify_control, &VerifyControl::on_test_line_changed));
 	m_verify_control->attach (m_test_line_button);
+	m_verify_control->set_focus_on_entry ();
 
 	/** Fetch out the side history and adjust accordingly
 	 */
@@ -583,7 +638,10 @@ DeckPlayer (VDeck& vdeck_) :
 
     m_uimanager = Gtk::UIManager::create ();
     m_uimanager->insert_action_group (m_actgroup);
+
+#ifndef IS_HILDON
     add_accel_group (m_uimanager->get_accel_group ());
+#endif
 
     try {
 		Glib::ustring ui_info =
@@ -631,7 +689,10 @@ DeckPlayer (VDeck& vdeck_) :
     ctrl_toolbar->set_toolbar_style (Gtk::TOOLBAR_ICONS);
     ctrl_toolbar->set_tooltips (true);
 
+#ifndef IS_HILDON
     set_has_separator (false);
+#endif
+
     Gtk::HBox* hbox = manage (new Gtk::HBox);
     hbox->pack_start (*ctrl_toolbar, true, true, 2);
     hbox->pack_start (*m_close_button, false, false, 0);
@@ -662,10 +723,20 @@ DeckPlayer (VDeck& vdeck_) :
 			mem_fun (*this, &DeckPlayer::on_key_pressed), false);
 	}
 	
+	/* For maemo, reduce overall font size
+	 */
+	Pango::FontDescription fd ("Bitstream Vera Sans 8");
+	m_shuffle_button->modify_font (fd);
+	m_edit_button->modify_font (fd);
+	m_vcard_edit_button->modify_font (fd);
+
 	/** Grand finale
 	 */
+#ifndef IS_HILDON
 	set_icon_from_file (GRAPPDATADIR "/pixmaps/deckplayer_32x32.png");
+#endif
     set_sensitivity ();
+	sensitize_controls ();
     m_deck.reset_progress ();
     set_win_name ();
     show_deck ();
@@ -676,6 +747,42 @@ DeckPlayer (VDeck& vdeck_) :
 //==============================================================================
 //                              Utilities
 //==============================================================================
+//
+#ifdef IS_HILDON
+
+gint
+DeckPlayer::
+run ()
+{
+    trace_with_mask("DeckPlayer::run",GUITRACE);
+
+	GRANULE->register_appview (this);
+	show ();
+	m_verify_control->set_focus_on_entry ();
+
+	DL ((APP,"Enter Main::run()\n"));
+	Gtk::Main::run ();
+
+	DL ((APP,"Exit Main::run()\n"));
+	return (Gtk::RESPONSE_NONE);
+}
+#endif
+
+void
+DeckPlayer::
+on_close_clicked ()
+{
+    trace_with_mask("DeckPlayer::on_close_clicked",GUITRACE);
+
+#ifdef IS_HILDON
+	GRANULE->unregister_appview (this);
+
+	DL ((GRAPP,"Call Main::quit()\n"));
+	Gtk::Main::quit ();
+#else
+	hide ();
+#endif
+}
 
 void
 DeckPlayer::
@@ -737,9 +844,8 @@ show_card (bool with_adjust_count_)
 	if (with_adjust_count_) {
 		adjust_count ();
 	}
-	/** set focus
-	 */
-	m_verify_control->grab_focus ();
+
+	m_verify_control->set_focus_on_entry ();
 }
 
 void
@@ -801,6 +907,37 @@ set_sensitivity ()
     }
 }
 
+void
+DeckPlayer::
+sensitize_controls ()
+{
+	trace_with_mask("DeckPlayer::set_sensitize", GUITRACE);
+
+    if (!m_is_real_deck) {
+		return;
+	}
+	DL ((GRAPP,"Deck is empty\n"));
+
+	if (m_deck.empty ()) {
+		m_shuffle_button    ->set_sensitive (false);
+		m_vcard_edit_button ->set_sensitive (false);
+
+		m_toolbar_go_first  ->set_sensitive (false);
+		m_toolbar_go_prev   ->set_sensitive (false);
+		m_toolbar_go_next   ->set_sensitive (false);
+		m_toolbar_go_last   ->set_sensitive (false);
+    }
+	else {
+		m_shuffle_button    ->set_sensitive (true);
+		m_vcard_edit_button ->set_sensitive (true);
+
+		m_toolbar_go_first  ->set_sensitive (true);
+		m_toolbar_go_prev   ->set_sensitive (true);
+		m_toolbar_go_next   ->set_sensitive (true);
+		m_toolbar_go_last   ->set_sensitive (true);
+	}
+}
+
 void 
 DeckPlayer::
 enable_answer_controls ()
@@ -940,13 +1077,6 @@ no_clue_cb ()
 	m_verify_control->clear ();
 }
 
-void
-DeckPlayer::
-on_close_clicked ()
-{
-    trace_with_mask("DeckPlayer::on_close_clicked",GUITRACE);
-    hide ();
-}
 
 void
 DeckPlayer::
@@ -979,7 +1109,8 @@ on_shuffle_clicked ()
 
     if (m_is_real_deck) {
 		random_shuffle (m_deck.begin (), m_deck.end ());
-		Gtk::MessageDialog qm (*this, 
+		/* Gtk::MessageDialog qm (*this,  */
+		Gtk::MessageDialog qm (
 				   _("Would you like to make cards\norder changes permanent?"),
 							   false,
                                Gtk::MESSAGE_QUESTION, 
@@ -1016,10 +1147,16 @@ on_edit_clicked ()
 	hide (); // hide DeckPlayer while DeckView is visible.
 
 	int idx = m_deck_iter == m_deck.end() ? -1 : (m_deck_iter - m_deck.begin());
-    DeckView dv (m_deck, idx);
-    dv.run ();
+	bool dv_modified = false;
+
+	{
+		DeckView dv (m_deck, idx);
+		dv.run ();
+		dv_modified = dv.is_modified ();
+	}
 
     DL((GRAPP,"Deck size after edit = %d cards\n", m_deck.size ()));
+	sensitize_controls ();
     /** 
      * Update viewer - iterators might have become invalid.
      */
@@ -1027,10 +1164,15 @@ on_edit_clicked ()
 		m_deck_iter = m_deck.end ();
 		DL ((APP,"m_deck_iter = end ()\n"));
 		show_card ();
+		
+#ifndef IS_HILDON
+		/** Under Hildon, this messes up return chain.
+		 */
 		on_close_clicked ();
+#endif
     }
     else {
-		if (dv.is_modified ()) {
+		if (dv_modified) {
 			DL ((APP,"Deck's been modified - go to start\n"));
 			show_deck ();
 		}
@@ -1071,18 +1213,29 @@ on_verify_clicked ()
 {
     trace_with_mask("DeckPlayer::on_verify_clicked",GUITRACE);
 
-	if (m_verify_control->get_state () == VC_ARMED) {
+	if (m_verify_control->get_state () == VC_ARMED) 
+	{
 		m_verify_control->compare (m_selected_side == FRONT ?
 								   m_back.get_text () :
 								   m_front.get_text ());
 	}
-	else if (m_verify_control->get_state () == VC_VALIDATED) {
-		go_next_cb ();
+	else if (m_verify_control->get_state () == VC_VALIDATED) 
+	{
+		if (m_is_real_deck) {
+			go_next_cb ();
+		}
+		else { /* CardBox Player*/ 
+			if (m_verify_control->matched ()) {
+				know_answer_cb ();
+			}
+			else {
+				no_clue_cb ();
+			}
+		}
 		m_verify_control->clear ();
 	}
 }
 
-
 /** 
   Key values can be found in /usr/include/gtk-2.0/gdk/gdkkeysyms.h
 
@@ -1120,15 +1273,47 @@ on_verify_clicked ()
 	| activate [Next] button  |                        |
     `-------------------------+------------------------/
 */
+/*
+  GDK_space     0x0020
+
+  GDK_Left      0xFF51
+  GDK_Up        0xFF52
+  GDK_Right     0xFF53
+  GDK_Down      0xFF54
+
+  GDK_KP_Left   0xFF96
+  GDK_KP_Up     0xFF97
+  GDK_KP_Right  0xFF98
+  GDK_KP_Down   0xFF99
+
+  GDK_Tab       0xFF09
+  GDK_Return    0xFF0D
+  GDK_KP_Enter  0xFF8D
+  GDK_F1        0xFFBE
+  GDK_F6        0xFFC3
+  GDK_F7        0xFFC4
+  GDK_F8        0xFFC5
+*/
+
 bool 
 DeckPlayer::
 on_key_pressed (GdkEventKey* evt_)
 {
     trace_with_mask("DeckPlayer::on_key_pressed",GUITRACE);
 
+#ifndef IS_HILDON
 	if (evt_->keyval == GDK_F1) {
 		MAINWIN->iconify ();
 	}
+#endif
+
+	if (evt_->keyval == GDK_space || evt_->keyval > 0xFF51) {
+		DL ((GRAPP,"Key pressed: 0x%04X, real_deck: %s, state: %s\n", 
+			 evt_->keyval, (m_is_real_deck ? "Yes" : "No"),
+			 (m_verify_control->get_state() == VC_START ? "START" :
+			  m_verify_control->get_state() == VC_ARMED ? "ARMED" : "VALIDATED")
+				));
+	}
 	
 	if (evt_->keyval == GDK_space && m_tab_activated) {
 		m_tab_activated = false;
@@ -1139,23 +1324,40 @@ on_key_pressed (GdkEventKey* evt_)
 		m_tab_activated = true;
 		return false;
 	}
-	
-	if (m_is_real_deck) {                                       // Real Deck
+
+	if (check_F6_key (evt_->keyval)) {
+		return true;
+	}
+
+	/*********************************************
+	 * Real Deck                                 *
+	 *********************************************/
+
+	if (m_is_real_deck) {                         
 		switch (m_verify_control->get_state ()) {
 
 		case VC_START:
 		case VC_VALIDATED:
-			if (evt_->keyval == GDK_Right  || evt_->keyval == GDK_KP_Right ||
-				evt_->keyval == GDK_Return || evt_->keyval == GDK_KP_Enter) {
+			if (evt_->keyval == GDK_Right    || 
+				evt_->keyval == GDK_KP_Right ||
+				evt_->keyval == GDK_Return   || 
+				evt_->keyval == GDK_KP_Enter ||
+				check_F7_key (evt_->keyval)) 
+			{
 				go_next_cb ();
 			}
-			else if (evt_->keyval == GDK_Left || evt_->keyval ==  GDK_KP_Left) {
+			else if (evt_->keyval == GDK_Left    || 
+					 evt_->keyval == GDK_KP_Left ||
+					 check_F8_key (evt_->keyval))
+			{
 				go_prev_cb ();
 			}
 			break;
 
 		case VC_ARMED:
-			if (evt_->keyval == GDK_Return || evt_->keyval == GDK_KP_Enter) {
+			if (evt_->keyval == GDK_Return || 
+				evt_->keyval == GDK_KP_Enter) 
+			{
 				m_verify_control->compare (m_selected_side == FRONT ?
 										   m_back.get_text () :
 										   m_front.get_text ());
@@ -1166,19 +1368,32 @@ on_key_pressed (GdkEventKey* evt_)
 			break;
 		}
 	}
-	else {                                                   // CardBox player
-		switch (m_verify_control->get_state ()) {
+	/*********************************************
+	 * CardBox Player                            *
+	 *********************************************/
+
+	else {
+		switch (m_verify_control->get_state ()) 
+		{
 		case VC_START:
-			if (evt_->keyval == GDK_Right || evt_->keyval == GDK_KP_Right) {
+			if (evt_->keyval == GDK_Right    || 
+				evt_->keyval == GDK_KP_Right ||
+				check_F7_key (evt_->keyval))
+			{
 				know_answer_cb ();
 			}
-			else if (evt_->keyval == GDK_Left || evt_->keyval ==  GDK_KP_Left) {
+			else if (evt_->keyval == GDK_Left    || 
+					 evt_->keyval == GDK_KP_Left ||
+					 check_F8_key (evt_->keyval))
+			{
 				no_clue_cb ();
 			}
 			break;
 
 		case VC_ARMED:
-			if (evt_->keyval == GDK_Return || evt_->keyval == GDK_KP_Enter) {
+			if (evt_->keyval == GDK_Return || 
+				evt_->keyval == GDK_KP_Enter) 
+			{
 				m_verify_control->compare (m_selected_side == FRONT ?
 										   m_back.get_text () :
 										   m_front.get_text ());
@@ -1189,7 +1404,9 @@ on_key_pressed (GdkEventKey* evt_)
 			break;
 
 		case VC_VALIDATED:
-			if (evt_->keyval == GDK_Return || evt_->keyval == GDK_KP_Enter) {
+			if (evt_->keyval == GDK_Return || 
+				evt_->keyval == GDK_KP_Enter) 
+			{
 				if (m_verify_control->matched ()) {
 					know_answer_cb ();
 				}
@@ -1226,14 +1443,6 @@ on_key_pressed (GdkEventKey* evt_)
 	return true;
 }	
 
-void
-DeckPlayer::
-on_template_clicked ()
-{
-    trace_with_mask("DeckPlayer::on_template_clicked",GUITRACE);
-
-}
-
 void
 DeckPlayer::
 set_win_name ()
@@ -1287,9 +1496,12 @@ repaint ()
    m_front  .modify_font (CONFIG->get_question_font ());
    m_back   .modify_font (CONFIG->get_answer_font ());
    m_example.modify_font (CONFIG->get_example_font ());
+   m_verify_control->repaint ();
 
+#ifndef IS_HILDON
    Gdk::Rectangle geom = CONFIG->get_deck_player_geometry ();
    resize (geom.get_width (), geom.get_height ());
+#endif
 }
 
 /**
