@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.cpp,v 1.35 2005/11/27 04:59:22 vlg Exp $
+// $Id: DeckView.cpp,v 1.41 2006/03/19 15:39:44 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.cpp
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004 by Vladislav Grinchenko 
+//  Copyright (c) 2004,2006 by Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -19,6 +19,8 @@
 #include <sstream>
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/stock.h>
+#include <gtkmm/buttonbox.h>
+
 #include <assa/AutoPtr.h>
 
 #include "Granule-main.h"
@@ -36,7 +38,11 @@ using sigc::bind;
 using sigc::mem_fun;
 
 DeckView::
-DeckView (VDeck& deck_, int selected_row_) :
+DeckView (VDeck& deck_, int selected_row_) 
+		:
+#ifdef IS_HILDON
+	Hildon::AppView ("Deck View"),
+#endif
 	m_add_button    (manage (new Gtk::Button (Gtk::StockID ("gtk-add")))),
 	m_edit_button   (
 		manage (new ButtonWithImageLabel (Gtk::Stock::DND, "Edit"))),
@@ -60,11 +66,25 @@ DeckView (VDeck& deck_, int selected_row_) :
 	trace_with_mask("DeckView::DeckView", GUITRACE);
 
 	set_title("Deck View");
-	set_resizable(true);
 	set_border_width (2);
+
+#ifdef IS_HILDON
+	/**
+	 * Not a dialog any more - it is a Bin (AppView) instead.
+	 * We handle packaging ourselves and mute all attributes of a 
+	 * Dialog everywhere down the line.
+	 */
+    m_vbox = manage (new Gtk::VBox (false, 2));
+	add (*m_vbox);
+
+	set_size_request (672, 396);
+//	set_transient_for (*HILDONAPPWIN);
+#else
+	set_resizable(true);
 	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
 	set_size_request (633, 410);	// width; height 
 	set_transient_for (*MAINWIN);
+#endif
 
 	Gtk::Label* label_total = manage (new Gtk::Label ("Total: "));
 	Gtk::VBox* vbox         = manage (new Gtk::VBox(false, 0));
@@ -119,8 +139,13 @@ DeckView (VDeck& deck_, int selected_row_) :
 
 	/** <Close> button
 	 */
+#ifdef IS_HILDON
+	m_close_button = manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, 
+													   "Close"));
+#else
 	m_close_button = add_button (Gtk::StockID ("gtk-close"), 
 								 Gtk::RESPONSE_CLOSE );
+#endif
 	m_close_button->set_flags  (Gtk::CAN_FOCUS);
 	m_close_button->set_relief (Gtk::RELIEF_NORMAL);
 
@@ -140,8 +165,10 @@ DeckView (VDeck& deck_, int selected_row_) :
 
 	while (iter != m_deck.end ()) {
 		row = *m_list_store_ref->append ();
-		row [m_columns.m_front_row] = trim_multiline ((*iter)->get_question ());
-		row [m_columns.m_back_row ] = trim_multiline ((*iter)->get_answer ());
+		row [m_columns.m_front_row] = 
+			Granule::trim_multiline ((*iter)->get_question ());
+		row [m_columns.m_back_row ] = 
+			Granule::trim_multiline ((*iter)->get_answer ());
 		row [m_columns.m_card     ] = *iter;
 		iter++;
 	}
@@ -218,7 +245,7 @@ DeckView (VDeck& deck_, int selected_row_) :
 	m_edit_button->signal_clicked ().connect (
 		mem_fun (*this, &DeckView::on_edit_clicked));
 	m_save_deck_button->signal_clicked ().connect (
-		mem_fun (*DECKMGR, &DeckManager::save_deck_cb));
+		mem_fun (*this, &DeckView::on_save_deck_clicked));
 	m_delete_button->signal_clicked ().connect (
 		mem_fun (*this, &DeckView::on_delete_clicked));
 	m_more_button->signal_clicked ().connect (
@@ -237,6 +264,22 @@ DeckView (VDeck& deck_, int selected_row_) :
 
 	get_vbox ()->pack_start (*h_viewbox);
 	get_vbox ()->pack_start (*h_infobox, Gtk::PACK_SHRINK, 4);
+
+#ifdef IS_HILDON
+	/**
+	 * Add separator and [Close] button in the ButtonBox
+	 */
+	Gtk::HSeparator* hseparator = manage (new Gtk::HSeparator);
+	get_vbox ()->pack_start (*hseparator, false, true, 0);
+
+	Gtk::HButtonBox*  hbuttonbox = manage (new Gtk::HButtonBox);
+	hbuttonbox->set_spacing (6);
+	hbuttonbox->set_border_width (6);
+
+	get_vbox ()->pack_start (*hbuttonbox, false, true, 0);
+	hbuttonbox->pack_end (*m_close_button, false, true, 0);
+#endif
+
 	set_cards_count ();
 
 	show_all ();
@@ -261,7 +304,10 @@ DeckView (VDeck& deck_, int selected_row_) :
 	}
 	set_sensitivity ();
 	set_win_name ();
+
+#ifndef IS_HILDON
 	set_icon_from_file (GRAPPDATADIR "/pixmaps/deckview_32x32.png");
+#endif
 
 }
 
@@ -283,6 +329,34 @@ set_cards_count ()
 //==============================================================================
 //                              Callbacks
 //------------------------------------------------------------------------------
+
+#ifdef IS_HILDON
+gint
+DeckView::
+run ()
+{
+    trace_with_mask("DeckView::run",GUITRACE);
+
+	GRANULE->register_appview (this);
+	Gtk::Main::run ();
+	return (Gtk::RESPONSE_NONE);
+}
+#endif
+
+void
+DeckView::
+on_close_clicked ()  
+{
+	trace_with_mask("DeckView::on_close_clicked", GUITRACE);
+
+#ifdef IS_HILDON
+	hide ();
+	GRANULE->unregister_appview (this);
+	DL ((GRAPP,"Call Main::quit()\n"));
+	Gtk::Main::quit ();
+#endif
+}
+
 void
 DeckView::
 on_column_sorted ()
@@ -321,8 +395,10 @@ add_new_card (Card* card_)
 	m_deck.push_back (card_);
 
 	Gtk::TreeModel::Row row = *m_list_store_ref->append ();
-	row [m_columns.m_front_row] = trim_multiline (card_->get_question ());
-	row [m_columns.m_back_row ] = trim_multiline (card_->get_answer ());
+	row [m_columns.m_front_row] = 
+		Granule::trim_multiline (card_->get_question ());
+	row [m_columns.m_back_row ] = 
+		Granule::trim_multiline (card_->get_answer ());
 	row [m_columns.m_card     ] = card_;
 	m_tree_sel_ref->select (row);
 	set_cards_count ();
@@ -370,13 +446,23 @@ on_edit_clicked ()
 	Gtk::TreeModel::Row row = *m_list_store_ref->children ()[m_row_idx];
 	VCard* card_iter = row [m_columns.m_card];
 
+#ifdef IS_HILDON
+	hide ();
+#endif
+
 	CardView cview (card_iter);
 	int ret = cview.run ();
 
 	if (ret == Gtk::RESPONSE_OK) {
-		row [m_columns.m_front_row] = trim_multiline(card_iter->get_question());
-		row [m_columns.m_back_row ] = trim_multiline(card_iter->get_answer());
+		row [m_columns.m_front_row] = 
+			Granule::trim_multiline(card_iter->get_question());
+		row [m_columns.m_back_row ] = 
+			Granule::trim_multiline(card_iter->get_answer  ());
 	}
+
+#ifdef IS_HILDON
+	show ();
+#endif
 }
 
 void 
@@ -427,8 +513,14 @@ on_more_clicked ()
 {
 	trace_with_mask("DeckView::on_more_clicked", GUITRACE);
 
+#ifdef IS_HILDON
+	DeckInfo deck_info (*HILDONAPPWIN, dynamic_cast<Deck&>(m_deck));
+	deck_info.run ();
+#else
 	DeckInfo deck_info (*this, dynamic_cast<Deck&>(m_deck));
 	deck_info.run ();
+#endif
+
 }
 
 void
@@ -479,9 +571,16 @@ on_card_up_clicked ()
 {
     trace_with_mask("DeckView::on_card_up_clicked",GUITRACE);
 
+	if (m_row_idx < 0) {
+		Gtk::MessageDialog em ("The deck is empty!",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
+	}
+
 	if (m_row_idx == 0) {
 		return;
 	}
+
 	children_t children = m_list_store_ref->children();
 	chiter_t a = children [m_row_idx];	
 
@@ -507,6 +606,12 @@ on_card_down_clicked ()
 {
     trace_with_mask("DeckView::on_card_down_clicked",GUITRACE);
 
+	if (m_row_idx < 0) {
+		Gtk::MessageDialog em ("The deck is empty!",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
+	}
+
 	if (m_row_idx == m_deck.size ()-1) {
 		return;
 	}
@@ -535,6 +640,12 @@ on_add_card_to_cardbox ()
 {
     trace_with_mask("DeckView::on_add_card_to_cardbox",GUITRACE);
 
+	if (m_row_idx < 0) {
+		Gtk::MessageDialog em ("The deck is empty!",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
+	}
+
 	if (m_deck.get_name ().find ("Untitled") != std::string::npos) {
 		Gtk::MessageDialog em1 ("Operation is not permitted!\n"
 								"Save the 'Untitled' Deck first.",
@@ -543,12 +654,6 @@ on_add_card_to_cardbox ()
 		return;
 	}
 
-	if (m_row_idx  < 0) {
-		Gtk::MessageDialog em2 ("The deck is empty!",Gtk::MESSAGE_ERROR);
-		em2.run ();
-		return;
-	}
-
 	VDeck::cardlist_iterator card_iter = m_deck.begin () + m_row_idx;
 	CARDBOX->add_card_to_box (
 		new CardRef (**card_iter, dynamic_cast<Deck&>(m_deck)));
@@ -637,24 +742,15 @@ set_win_name ()
 	set_title ("Deck View : " + title);
 }
 
-/**
- * For multiline entries (starting with v 1.1.5), 
- * remove part of the string following first newline character.
- * But first, strip possible Pango markup. Markup can span across
- * multiple lines and interpreting only first line might result
- * in invalid syntax.
- */
-Glib::ustring
+void
 DeckView::
-trim_multiline (const Glib::ustring& orig_text_)
+on_save_deck_clicked ()
 {
-	gchar* pc = Granule::strip_pango_markup (orig_text_.c_str ());
-	Glib::ustring text_ = pc;
-	g_free (pc);
-	Glib::ustring::size_type idx  = text_.find_first_of ("\n");
-	Glib::ustring result (text_);
-	if (idx != std::string::npos) {
-		result.replace (idx, text_.size (), "");
+	if (m_row_idx < 0) {
+		Gtk::MessageDialog em ("The deck is empty!",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
 	}
-	return result;
+
+	DECKMGR->save_deck_cb ();
 }
