@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: GeometryPrefs.cpp,v 1.8 2005/06/19 04:15:36 vlg Exp $
+// $Id: GeometryPrefs.cpp,v 1.10 2006/03/19 15:39:44 vlg Exp $
 //------------------------------------------------------------------------------
 //                            GeometryPrefs.cpp
 //------------------------------------------------------------------------------
@@ -18,6 +18,7 @@
 
 #include <gtkmm/frame.h>
 #include <gtkmm/adjustment.h>
+#include <gtkmm/scrolledwindow.h>
 
 #include "Granule-main.h"
 #include "Config.h"
@@ -268,9 +269,33 @@ GeometryPrefs (PrefWindow& pwin_) :
 	hbox->pack_start (*m_table,     Gtk::PACK_SHRINK, 4);
 	hbox->pack_start (*post_hidden, Gtk::PACK_EXPAND_WIDGET);
 
+
+	/** Maemo has a narrow screen and thus requires
+	 * 	smart packing.
+	 */
+#ifdef IS_HILDON
+
+	Gtk::ScrolledWindow* scrollw = Gtk::manage (new Gtk::ScrolledWindow);
+	scrollw->set_flags (Gtk::CAN_FOCUS);
+	scrollw->set_shadow_type (Gtk::SHADOW_NONE);
+	scrollw->set_policy (Gtk::POLICY_AUTOMATIC , Gtk::POLICY_ALWAYS);
+	scrollw->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
+
+	Gtk::VBox* scrollw_vbox = Gtk::manage (new Gtk::VBox (false, 0));
+
+	scrollw_vbox->pack_start (*hbox,          Gtk::PACK_SHRINK, 0);
+	scrollw_vbox->pack_start (*options_frame, Gtk::PACK_SHRINK, 0);
+
+	scrollw->add (*scrollw_vbox);
+	pack_start (*scrollw);
+
+#else  // Desktop
+
 	pack_start (*hbox,          Gtk::PACK_SHRINK, 0);
 	pack_start (*options_frame, Gtk::PACK_SHRINK, 0);
 
+#endif
+
 	/** Set callbacks
 	 */
 	m_aspect->get_checkbox ()->signal_toggled ().connect (
