@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.h,v 1.6 2005/01/17 04:19:46 vlg Exp $
+// $Id: PrefWindow.h,v 1.9 2006/03/19 15:39:44 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
@@ -31,7 +31,7 @@
 class PrefWindow : public MyPropertyBox
 {
 public:
-    PrefWindow (Gtk::Window& parent_);
+    PrefWindow (Gtk::Widget& parent_);
     ~PrefWindow();
     
     bool run ();
@@ -70,6 +70,9 @@ stop ()
 {
     trace_with_mask("PrefWindow::stop",GUITRACE);
     hide ();
+#ifdef IS_HILDON
+	GRANULE->unregister_appview (this);
+#endif
     Gtk::Main::quit ();
 }
 
