@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              TopMenuBar.cpp
 //------------------------------------------------------------------------------
-// $Id: TopMenuBar.cpp,v 1.21 2005/10/13 03:04:13 vlg Exp $
+// $Id: TopMenuBar.cpp,v 1.25 2006/03/19 15:39:44 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004 by Vladislav Grinchenko
 //
@@ -39,29 +39,35 @@ create ()
     m_open_recent_menu = manage (new Menu ());
     {
 		using namespace Menu_Helpers;
-		/**--------------------------------------------------------------
-		 *  [File] Menu
-		 */
+		/**-------------**
+		 *  [File] Menu  *
+		 **-------------**/
 		m_file_menu = manage (new Menu ());
 		MenuList& f = m_file_menu->items ();
 
 		f.push_back (MenuElem ("_Preferences", 
 							   mem_fun (m_mw, &MainWindow::pref_cb)));
 		m_preferences = &f.back ();
+#ifndef IS_HILDON
 		f.push_back (SeparatorElem ());
 		f.push_back (MenuElem ("_Exit", 
 							   mem_fun (m_mw, &MainWindow::exit_cb)));
 		m_exit = &f.back ();
+#endif
 
 		// Assemble (backwards?)
+#ifdef IS_HILDON	
+		MenuList& list_bar = m_mw.get_menu ()->items ();
+#else
 		MenuList& list_bar = m_menu_bar->items ();
+#endif
 
 		list_bar.push_back (MenuElem ("_File", 
 									  Gtk::AccelKey("<control>f"), 
 									  *m_file_menu));
-		/**-----------------
-		 *  [CardFile] Menu
-		 */
+		/**-----------------**
+		 *  [CardFile] Menu  *
+		 **-----------------**/
 		m_cardfile_menu = manage (new Menu ());
 		MenuList& cf = m_cardfile_menu->items ();
 
@@ -79,19 +85,33 @@ create ()
 		cf.push_back (MenuElem ("_Save", 
 								mem_fun (m_mw, &MainWindow::save_cb)));
 		m_save = &cf.back ();
+
 		cf.push_back (MenuElem ("Save _As", 
 								mem_fun(m_mw, &MainWindow::save_as_cb)));
 		m_save_as = &cf.back ();
+
 		cf.push_back (SeparatorElem ());
 		cf.push_back (MenuElem("_Close", 
 							   mem_fun(m_mw, &MainWindow::close_cb)));
 		m_close = &cf.back ();
+		/*------------------------------*/
 		cf.push_back (SeparatorElem ());
 
-		/**----------------------------
-		 *  [CardFile]->[Play] SubMenu
-		 *-----------------------------
-		 */
+		/**------------------------------**
+		 *  [CardFile]->[Export] SubMenu  *
+		 **------------------------------**/
+		m_export_menu = manage (new Menu ());
+		MenuList& expt = m_export_menu->items ();
+
+		expt.push_back (MenuElem ("Sound Bites",
+								  mem_fun(m_mw, &MainWindow::export_sound_cb)));
+
+		cf.push_back (MenuElem ("_Export", *m_export_menu));
+		m_export = &cf.back ();
+
+		/**----------------------------**
+		 *  [CardFile]->[Play] SubMenu  *
+		 **----------------------------**/
 		m_play_menu = manage (new Menu ());
 		MenuList& cfp = m_play_menu->items ();
 
@@ -105,14 +125,13 @@ create ()
 								*m_play_menu));
 		m_play_in = &cf.back ();
 
-		/**------------------------------------------------------------------*/
 		list_bar.push_back (MenuElem ("_CardFile", 
 									  Gtk::AccelKey("<control>c"), 
 									  *m_cardfile_menu));
-		/**---------------------------
-		 *  Menu items for [Deck] Menu
-		 *----------------------------
-		 */
+
+		/**----------------------------**
+		 *  Menu items for [Deck] Menu  *
+		 **----------------------------**/
 		m_deck_menu = manage (new Menu ());
 		MenuList& d = m_deck_menu->items ();
 
@@ -150,11 +169,11 @@ create ()
 							   mem_fun (m_dm, &DeckManager::play_deck_cb)));
 		m_deck_play = &d.back ();
 
-		/**----------------------------------
-		 *  [Deck]->[Add2Box] SubMenu
-		 */
-		m_add2box_menu = manage (new Menu ());
-		MenuList& a2bp = m_add2box_menu->items ();
+		/**------------------------------**
+		 *  [Deck]->[Add_To_Box] SubMenu  *
+		 **------------------------------**/
+		m_add_to_box_menu = manage (new Menu ());
+		MenuList& a2bp = m_add_to_box_menu->items ();
 
 		for (int j = 0; j < CARD_BOX_SIZE; j++) {
 			std::ostringstream os;
@@ -163,7 +182,8 @@ create ()
 						   bind<int>(mem_fun(m_dm, 
 									 &DeckManager::add_deck_to_box), j)));
 		}
-		d.push_back (MenuElem ("Add to ", *m_add2box_menu));
+		d.push_back (MenuElem ("Add to ", *m_add_to_box_menu));
+		m_add_to_box = &d.back ();
 
 		/*------------------------------------------------------------*/
 		list_bar.push_back (MenuElem ("_Deck", 
@@ -173,20 +193,26 @@ create ()
 							   mem_fun (m_mw, &MainWindow::unselect_deck_cb)));
 		m_deck_unselect = &d.back ();
 
-		/**--------------------------------------------------------------*/
+		/**-----------------------------------------------------------*/
 		Menu* help_menu = manage (new Menu ());
         MenuList& help_list = help_menu->items ();
 
         help_list.push_back (MenuElem ("_About",
                                        mem_fun (m_mw, &MainWindow::about_cb)));
 
-		/**--------------------------------------------------------------
+		/**-------------------
 		   Assemble backwards
 		 */
 		list_bar.push_back (
 			MenuElem ("_Help", Gtk::AccelKey("<control>h"), *help_menu));
 		list_bar.back ().set_right_justified ();
 
+#ifdef IS_HILDON
+		list_bar.push_back (MenuElem ("_Exit", 
+							mem_fun (m_mw, &MainWindow::exit_cb)));
+		m_exit = &f.back ();
+#endif
+
     }
 }
 
@@ -217,13 +243,26 @@ refill_history_list ()
     Gtk::Menu_Helpers::MenuList& file_list = m_open_recent_menu->items ();
     file_list.clear ();
 
+	std::string underscored;
     const Config::DHList& dh_list = CONFIG->get_document_history ();
     Config::DHList_CIter cit = dh_list.begin ();
 
-    while (cit != dh_list.end ()) {
+	while (cit != dh_list.end ()) {
+		if ((*cit).find ("_") == std::string::npos) {
+			underscored = (*cit);
+		}
+		else {
+			std::string::const_iterator pos = (*cit).begin ();
+			underscored = "";
+			while (pos != (*cit).end ()) {
+				if ((*pos) == '_') { underscored += "__"; }
+				else { underscored += (*pos); }
+				pos++;
+			}
+		}
 		file_list.push_back (
 			Gtk::Menu_Helpers::MenuElem (
-				(*cit).c_str (),
+				underscored,
 				bind<guint> (
 					mem_fun (m_mw, &MainWindow::open_recent_cb), position)));
 		position++;
@@ -257,6 +296,7 @@ set_sensitivity (WMState state_)
 		m_open_recent->set_sensitive (true ); // open recent
 		m_save       ->set_sensitive (false); // save
 		m_save_as    ->set_sensitive (false); // save as
+		m_export     ->set_sensitive (false); // export
 		m_close      ->set_sensitive (false); // close
 		m_exit       ->set_sensitive (true ); // exit
 		m_play_in    ->set_sensitive (false); // play in CardBox
@@ -268,6 +308,7 @@ set_sensitivity (WMState state_)
 		m_open_recent->set_sensitive (false); // open recent
 		m_save       ->set_sensitive (true ); // save
 		m_save_as    ->set_sensitive (true ); // save as
+		m_export     ->set_sensitive (true ); // export
 		m_close      ->set_sensitive (true ); // close
 		m_exit       ->set_sensitive (true ); // exit
 		m_play_in    ->set_sensitive (true ); // play in CardBox
@@ -279,6 +320,7 @@ set_sensitivity (WMState state_)
 		m_open_recent->set_sensitive (false); // open recent
 		m_save       ->set_sensitive (false); // save
 		m_save_as    ->set_sensitive (true ); // save as
+		m_export     ->set_sensitive (true ); // export
 		m_close      ->set_sensitive (true ); // close
 		m_exit       ->set_sensitive (true ); // exit
 		m_play_in    ->set_sensitive (true ); // play in CardBox
@@ -301,7 +343,7 @@ set_deck_sensitivity (bool is_deck_selected_, size_t num_open_decks_)
 		m_deck_save_as  ->set_sensitive (true );
 		m_deck_save_all ->set_sensitive (false);
 		m_deck_play     ->set_sensitive (true );
-		m_add2box_menu  ->set_sensitive (true );
+		m_add_to_box    ->set_sensitive (true );
 		m_deck_unselect ->set_sensitive (true );
 	}
 	else {
@@ -313,7 +355,7 @@ set_deck_sensitivity (bool is_deck_selected_, size_t num_open_decks_)
 		m_deck_save_as  ->set_sensitive (false);
 		m_deck_save_all ->set_sensitive ((num_open_decks_ > 0));
 		m_deck_play     ->set_sensitive (false);
-		m_add2box_menu  ->set_sensitive (false);
+		m_add_to_box    ->set_sensitive (false);
 		m_deck_unselect ->set_sensitive (false);
 	}
 }
